import 'dart:math';
import 'package:admob_flutter/admob_flutter.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:simple_fontellico_progress_dialog/simple_fontico_loading.dart';
import 'package:unity_ads_plugin/unity_ads_plugin.dart';
import 'custom/custom_interstitial_ads.dart';
import 'enum/ads_platform_enum.dart';
import 'model/admob_settings.dart';

class InterstitialAds {
  static SimpleFontelicoProgressDialog? _dialog;

  static _showDialog(BuildContext context,
      SimpleFontelicoProgressDialogType type, String text) async {
    _dialog ??= SimpleFontelicoProgressDialog(
          context: context, barrierDimisable: false);

    _dialog?.show(
        message: "${_admobSettings!.adsLoadingText}",
        type: type,
        backgroundColor: Colors.yellow,
        width: 150.0,
        height: 105.0,
        radius: 10,
        indicatorColor: Colors.black,
        loadingIndicator: Text(
          '${_admobSettings!.adsLoadingText}',
          style: const TextStyle(fontSize: 40),
        ));
  }

  static AdmobSettings? _admobSettings;
  static bool? _isTestAds;

  static void init(AdmobSettings admobSettings, bool isTestAds) {
    _admobSettings = admobSettings;
    _isTestAds = isTestAds;
    _initInterstitial();
  }

  static AdmobInterstitial? _interstitialAd;

  /**
   * It will use to init all ads.
   */
  static _initInterstitial() {

    /// init unity ads for both banner and interstitial
    UnityAds.init(
      gameId: "${_admobSettings!.unityAds!.adsAppIdAndroid}",
      testMode: _isTestAds!,
    );

    /// init admob interstitial ads.
    _interstitialAd = AdmobInterstitial(
      adUnitId: _isTestAds!? '/6499/example/interstitial':_admobSettings!.googleAds!.bannerAdUnitIdAndroid!,
      listener: (AdmobAdEvent event, Map<String, dynamic>? args) {
        if (event == AdmobAdEvent.closed)
        {
          _interstitialAd?.load();
        }
        else if (event == AdmobAdEvent.failedToLoad)
          {
            if (_admobSettings!.unityAds!.isInterstitialAds == true && _admobSettings!.unityAds!.visibilityPercentage! > 0) {
              UnityAds.showVideoAd(
                  placementId: 'Interstitial_Android',
                  onFailed: (placementId, error, message) {
                    if (_admobSettings!.customAds!.isInterstitialAds == true && _admobSettings!.customAds!.visibilityPercentage! > 0) {
                      Get.to(
                          CustomInterstitialAds(admobSettings: _admobSettings,),
                          preventDuplicates: false);
                    }
                  });
            }
            else
            {
              if (_admobSettings!.customAds!.isInterstitialAds == true && _admobSettings!.customAds!.visibilityPercentage! > 0) {
                Get.to(CustomInterstitialAds(admobSettings: _admobSettings),
                    preventDuplicates: false);
              }
            }
          }
      },      
    );

  }


  static Future showInterstitialAds(context) async 
  {

    int googleAdsRange = _admobSettings!.googleAds!.visibilityPercentage!;
    int unityAdsPackageRange =
        googleAdsRange + _admobSettings!.unityAds!.visibilityPercentage!;
    int customAdsRange = unityAdsPackageRange +
        _admobSettings!.customAds!.visibilityPercentage!; // total range

    /// Default selection.
    AdsPlatformEnum currentAdsPlatformEnum = AdsPlatformEnum.NONE;

    /// find random range for ads.
    Random rnd = Random();
    int randomNumber = rnd.nextInt(customAdsRange);

    if(_admobSettings!.isInterstitialAds == true) {
      if (randomNumber <= googleAdsRange) {
        if (_admobSettings!.googleAds!.isInterstitialAds == true && _admobSettings!.googleAds!.visibilityPercentage! > 0) {
          currentAdsPlatformEnum = AdsPlatformEnum.GOOGLE_ADMOB;
        } else if (_admobSettings!.unityAds!.isInterstitialAds == true && _admobSettings!.unityAds!.visibilityPercentage! > 0) {
          currentAdsPlatformEnum = AdsPlatformEnum.UNITY_ADS;
        } else if (_admobSettings!.customAds!.isInterstitialAds == true && _admobSettings!.customAds!.visibilityPercentage! > 0) {
          currentAdsPlatformEnum = AdsPlatformEnum.CUSTOME_ADS;
        }
      }
      else if (randomNumber > googleAdsRange &&
          randomNumber <= unityAdsPackageRange) {
        if (_admobSettings!.unityAds!.isInterstitialAds == true && _admobSettings!.unityAds!.visibilityPercentage! > 0) {
          currentAdsPlatformEnum = AdsPlatformEnum.UNITY_ADS;
        } else if (_admobSettings!.googleAds!.isInterstitialAds == true && _admobSettings!.googleAds!.visibilityPercentage! > 0) {
          currentAdsPlatformEnum = AdsPlatformEnum.GOOGLE_ADMOB;
        } else if (_admobSettings!.customAds!.isInterstitialAds == true && _admobSettings!.customAds!.visibilityPercentage! > 0) {
          currentAdsPlatformEnum = AdsPlatformEnum.CUSTOME_ADS;
        }
      }
      else if (randomNumber > unityAdsPackageRange &&
          randomNumber <= customAdsRange) {
        if (_admobSettings!.customAds!.isInterstitialAds == true && _admobSettings!.customAds!.visibilityPercentage! > 0) {
          currentAdsPlatformEnum = AdsPlatformEnum.CUSTOME_ADS;
        } else if (_admobSettings!.googleAds!.isInterstitialAds == true && _admobSettings!.googleAds!.visibilityPercentage! > 0) {
          currentAdsPlatformEnum = AdsPlatformEnum.GOOGLE_ADMOB;
        } else if (_admobSettings!.unityAds!.isInterstitialAds == true && _admobSettings!.unityAds!.visibilityPercentage! > 0) {
          currentAdsPlatformEnum = AdsPlatformEnum.UNITY_ADS;
        }
      }
    }

    /// It will show interstitial ads as on priority.
    int timerLoader = _admobSettings!.adsLoadingDelay!;
    if (currentAdsPlatformEnum == AdsPlatformEnum.GOOGLE_ADMOB) {
      _showDialog(context, SimpleFontelicoProgressDialogType.hurricane,
          'Hurricane');
      Future.delayed(Duration(milliseconds: timerLoader), () async
      {
        bool? isLoaded = await _interstitialAd!.isLoaded;
        if(isLoaded!) {
          _interstitialAd!.show();
        }else
          {
            _interstitialAd!.load();
          }
      }).then((value) {
        Get.back();
      });
    }else if (currentAdsPlatformEnum == AdsPlatformEnum.UNITY_ADS) {
      _showDialog(context, SimpleFontelicoProgressDialogType.hurricane,
          'Hurricane');
      Future.delayed(Duration(milliseconds: timerLoader), () {
        UnityAds.showVideoAd(
            placementId: 'Interstitial_Android',
            onFailed: (placementId, error, message) {
              if (_admobSettings!.customAds!.isInterstitialAds == true && _admobSettings!.customAds!.visibilityPercentage! > 0) {
                Get.to(CustomInterstitialAds(admobSettings: _admobSettings),
                    preventDuplicates: false);
              }
            });
      }).then((value) => Get.back());
    }else if (currentAdsPlatformEnum == AdsPlatformEnum.CUSTOME_ADS) {
      _showDialog(context, SimpleFontelicoProgressDialogType.hurricane,
          'Hurricane');
      Future.delayed(Duration(milliseconds: timerLoader), () {
        Get.back();
      }).then((value) => Get.to(CustomInterstitialAds(admobSettings: _admobSettings,),preventDuplicates: false));
    }
  }
}
